/**
  * udpecho.c - simple fast udp sender without timeouts
  * Copyright (C) 2006  NETWAYS GmbH, Marius Hein
  * $Id: udpecho.c 1355 2006-08-17 08:04:31Z mstreb $
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  * ----> To compile: gcc -Wall udpecho.c -o udpecho <----
  *
  **/

/**
 *
 * CHANGELOG:
 *
 * 1.0
 * 	- INITIAL RELEASE
 *
 * 1.1
 * -	[netways #15679]: NagiosGrapher-1.6-rc1 udpecho.c: Send buffer too small for plugin output
 * 		Increased the buffersize to 512 for larger pluginoutput (Ed Voncken)
 *
 **/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdio.h>

#define DEF_PORT 5667

void die(char* msg);
void p_usage();

int main(int argc, char* argv[]) {
	const int buffer_length = 512;

	int sockfd, n;
	int port=0;
	struct sockaddr_in server;
	char buffer[buffer_length];
	char recv[buffer_length];

	if (argc < 3) {
		p_usage();
	}

	sockfd = socket(AF_INET, SOCK_DGRAM, 0);
	if (sockfd <= 0) {
		die("Can't create socket!");
	}

	if (argv[3] && argv[3] > 0) {
		port = atoi(argv[3]);
	}
	else {
		port = DEF_PORT;
	}

	bzero(&server, sizeof(server));
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = inet_addr(argv[1]);
	server.sin_port = htons(port);

	strcpy(buffer, argv[2]);
	n = sendto(sockfd, buffer, strlen(buffer), 0, (struct sockaddr *)&server, sizeof(server));
	if (!n == strlen(buffer)) {
		die("Wrong size of bytes!");
	}

	/*
	 *
	 * If you want to see what the server is returning ... (maybe this needs to much time)
	 *
		n=0;
		n = recvfrom(sockfd, recv, 10000, 0, NULL, NULL);
		if (n>0) {
			recv[n] = 0;
			printf("%s: %s\n", argv[0], recv);
		}
	*/

	close (sockfd);

	bzero(buffer, buffer_length);
	bzero(recv, buffer_length);

	return (0);
}

void die(char* msg) {
	printf("ERROR: %s\n", msg);

	exit (-1);
}

void p_usage() {
	printf("Usage: udpecho <ip> <message> [port]\n");

	exit (0);
}
